#pragma once
#ifndef SOFTWARE_VERSION_H
#define SOFTWARE_VERSION_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

#ifdef __GNUC__
#undef major
#undef minor
#endif

#include <string>
#include <unordered_map>
#include <boost/algorithm/string.hpp>

enum enum_software_version_type
{
    svt_unknown                      = 255,
    svt_pze                          = 0,
    svt_zk                           = 1,
    svt_bde                          = 2,
    svt_mobil                        = 3,
    svt_mde                          = 4,
    svt_ae                           = 5,
    svt_timeboy                      = 6,
    svt_exklusive                    = 7,
    svt_flex                         = 8,
    svt_timeboy_mobil_pze            = 9,
    svt_evo_2_8                      = 10,
    svt_evo_4_3                      = 11,
    svt_extension_a                  = 12,
    svt_extension_b                  = 13,
    svt_extension_c                  = 14,
    svt_io_box                       = 15,
    svt_zk_box                       = 16,
    svt_mobil_box                    = 17,
    svt_timeboy_tbdock               = 18,
    svt_mobil_tbdock                 = 19,
    svt_zk_knoten                    = 20,
    svt_evo_3_5_universal            = 21,
    svt_sams_ocl                     = 22,
    svt_evo_3_5                      = 23,
    svt_zk_reader_plus               = 24,
    svt_zk_zentrale                  = 25
    // Die Bezeichnung muss in die Tabelle TypeDeviceNameValue::m_type_device_name_value eingetragen werden.
};

class TypeDeviceNameValue
{
public:
    typedef std::unordered_map<std::string, std::pair<std::string, enum_software_version_type> > type_device_name_value;

public:
    TypeDeviceNameValue()
    {
        // Inhalt der Map ist: map<lower(Name), pair<Wert, Name> >
        auto make_device_name_value = [this]( const std::string& name, enum_software_version_type val ) -> std::pair<std::string, std::pair<std::string, enum_software_version_type> > 
        {
            return std::make_pair( boost::to_lower_copy( name ), std::make_pair( name, val ) );
        };
        // Liste der Geraetebezeichnung zum Vergleich mittels Kleinbuchstaben gemappt auf die Geraetetypnummer und abgesprochene Namensdarstellung.
        m_type_device_name_value.insert( make_device_name_value( "?", svt_unknown ) );
        m_type_device_name_value.insert( make_device_name_value( "PZE", svt_pze ) );
        m_type_device_name_value.insert( make_device_name_value( "ZK", svt_zk ) );
        m_type_device_name_value.insert( make_device_name_value( "BDE", svt_bde ) );
        m_type_device_name_value.insert( make_device_name_value( "MOBIL", svt_mobil ) );
        m_type_device_name_value.insert( make_device_name_value( "MDE", svt_mde ) );
        m_type_device_name_value.insert( make_device_name_value( "AE", svt_ae ) );
        m_type_device_name_value.insert( make_device_name_value( "TIMEBOY", svt_timeboy ) );
        m_type_device_name_value.insert( make_device_name_value( "EXKLUSIV", svt_exklusive ) );
        m_type_device_name_value.insert( make_device_name_value( "Flex", svt_flex ) );
        m_type_device_name_value.insert( make_device_name_value( "TB-MOBIL-PZE", svt_timeboy_mobil_pze ) );
        m_type_device_name_value.insert( make_device_name_value( "Evo28", svt_evo_2_8 ) );
        m_type_device_name_value.insert( make_device_name_value( "Evo43", svt_evo_4_3 ) );
        m_type_device_name_value.insert( make_device_name_value( "VARIO", svt_extension_a ) );
        m_type_device_name_value.insert( make_device_name_value( "IPC", svt_extension_b ) );
        m_type_device_name_value.insert( make_device_name_value( "Q7", svt_extension_c ) );
        m_type_device_name_value.insert( make_device_name_value( "IOBOX", svt_io_box ) );
        m_type_device_name_value.insert( make_device_name_value( "ZKBOX", svt_zk_box ) );
        m_type_device_name_value.insert( make_device_name_value( "MOBILBOX", svt_mobil_box ) );
        m_type_device_name_value.insert( make_device_name_value( "DockingV2", svt_timeboy_tbdock ) );
        m_type_device_name_value.insert( make_device_name_value( "LoggerV2", svt_mobil_tbdock ) );
        m_type_device_name_value.insert( make_device_name_value( "ZKKNOTEN", svt_zk_knoten ) );
        m_type_device_name_value.insert( make_device_name_value( "EVO35U", svt_evo_3_5_universal ) );
        m_type_device_name_value.insert( make_device_name_value( "SAMS-OCL", svt_sams_ocl ) );
        m_type_device_name_value.insert( make_device_name_value( "EVO35", svt_evo_3_5 ) );
        m_type_device_name_value.insert( make_device_name_value( "ZKREADERPLUS", svt_zk_reader_plus ) );
        m_type_device_name_value.insert( make_device_name_value( "ZKZENTRALE", svt_zk_zentrale ) );
    }

public:
    std::string get_device_name( enum_software_version_type type );
    enum_software_version_type get_device_value( const std::string& str );

private:
    type_device_name_value m_type_device_name_value;
};

class SoftwareVersion
{
public:
    SoftwareVersion();
    SoftwareVersion( const std::string& version );
    SoftwareVersion( const char* version );
    SoftwareVersion( unsigned int version );
    SoftwareVersion( unsigned char master, char major, unsigned char minor, unsigned char build, enum_software_version_type type_device = svt_unknown );
    virtual ~SoftwareVersion();
    friend class SoftwarebuildVersion;

private:
    std::string m_version;
    unsigned char m_master;
    unsigned char m_major;
    unsigned char m_minor;
    unsigned char m_build;
    enum_software_version_type m_type_device;

public:
    void reset();
    bool parse( const std::string& version );
    bool parse( unsigned int version );

    inline unsigned char master()
    {
        return m_master;
    }
    inline unsigned char major()
    {
        return m_major;
    }
    inline unsigned char minor()
    {
        return m_minor;
    }
    inline unsigned char build()
    {
        return m_build;
    }
    inline enum_software_version_type typeDevice()
    {
        return m_type_device;
    }

public: // Zuweisungsoperator
    SoftwareVersion& operator=( const char* version )
    {
        if ( this->parse( version ) == false )
        {
            this->reset();
        }

        return *this;
    }
    SoftwareVersion& operator=( unsigned int version )
    {
        if ( this->parse( version ) == false )
        {
            this->reset();
        }

        return *this;
    }

public: // Vergleichsoperatoren
    // Gleichheit
    bool operator==( SoftwareVersion& version )
    {
        if ( this->m_master == version.m_master 
            && this->m_major == version.m_major 
            && this->m_minor == version.m_minor )
        {
            return true;
        }

        return false;
    }
    bool operator==( const char* version )
    {
        SoftwareVersion lVersion( version );
        return (*this == lVersion);
    }
    // Ungleichheit
    bool operator!=( SoftwareVersion& version )
    {
        if ( this->m_master != version.m_master 
            || this->m_major != version.m_major 
            || this->m_minor != version.m_minor )
        {
            return true;
        }

        return false;
    }
    bool operator!=( const char* version )
    {
        SoftwareVersion lVersion( version );
        return (*this != lVersion);
    }
    // Groesser
    bool operator>( SoftwareVersion& version )
    {
        if ( this->m_master > version.m_master 
            || (this->m_master == version.m_master && this->m_major > version.m_major) 
            || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor > version.m_minor) )
        {
            return true;
        }

        return false;
    }
    bool operator>( const char* version )
    {
        SoftwareVersion lVersion( version );
        return (*this > lVersion);
    }
    // Groesser gleich
    bool operator>=( SoftwareVersion& version )
    {
        if ( *this > version || *this == version )
        {
            return true;
        }

        return false;
    }
    bool operator>=( const char* version )
    {
        SoftwareVersion lVersion( version );
        return (*this >= lVersion);
    }
    // Kleiner
    bool operator<( SoftwareVersion& version )
    {
        if ( this->m_master < version.m_master 
            || (this->m_master == version.m_master && this->m_major < version.m_major) 
            || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor < version.m_minor) )
        {
            return true;
        }

        return false;
    }
    bool operator<( const char* version )
    {
        SoftwareVersion lVersion( version );
        return (*this < lVersion);
    }
    // Kleiner gleich
    bool operator<=( SoftwareVersion& version )
    {
        if ( *this < version || *this == version )
        {
            return true;
        }

        return false;
    }
    bool operator<=( const char* version )
    {
        SoftwareVersion lVersion( version );
        return (*this <= lVersion);
    }

public: // Cast operatoren
    operator const char* ()
    {
        return m_version.c_str();
    }
    operator unsigned int ()
    {
        return static_cast<unsigned int>( (m_master << 24) | (m_major << 16) | (m_minor << 8) | m_build );
    }
};

class SoftwarebuildVersion : public SoftwareVersion
{
public:
    SoftwarebuildVersion() : SoftwareVersion()
    {

    }
    SoftwarebuildVersion( const std::string& version ) 
        : SoftwareVersion( version )
    {

    }
    SoftwarebuildVersion( const char* version ) 
        : SoftwareVersion( version )
    {

    }
    SoftwarebuildVersion( unsigned int version ) 
        : SoftwareVersion( version )
    {

    }
    SoftwarebuildVersion( unsigned char master, char major, unsigned char minor, unsigned char build, enum_software_version_type type_device = svt_unknown ) 
        : SoftwareVersion( master, major, minor, build, type_device )
    {
    }

public: // Vergleichsoperatoren
    // Gleichheit
    bool operator==( SoftwarebuildVersion& version )
    {
        if ( this->m_master == version.m_master 
            && this->m_major == version.m_major 
            && this->m_minor == version.m_minor 
            && this->m_build == version.m_build)
        {
            return true;
        }

        return false;
    }
    bool operator==( const char* version )
    {
        SoftwarebuildVersion lVersion( version );
        return (*this == lVersion);
    }
    // Ungleichheit
    bool operator!=( SoftwarebuildVersion& version )
    {
        if ( this->m_master != version.m_master 
            || this->m_major != version.m_major 
            || this->m_minor != version.m_minor 
            || this->m_build != version.m_build)
        {
            return true;
        }

        return false;
    }
    bool operator!=( const char* version )
    {
        SoftwarebuildVersion lVersion( version );
        return (*this != lVersion);
    }
    // Groesser
    bool operator>( SoftwarebuildVersion& version )
    {
        if ( this->m_master > version.m_master 
            || (this->m_master == version.m_master && this->m_major > version.m_major) 
            || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor > version.m_minor) 
            || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor == version.m_minor && this->m_build > version.m_build) )
        {
            return true;
        }

        return false;
    }
    bool operator>( const char* version )
    {
        SoftwarebuildVersion lVersion( version );
        return (*this > lVersion);
    }
    // Groesser gleich
    bool operator>=( SoftwarebuildVersion& version )
    {
        if ( *this > version || *this == version )
        {
            return true;
        }

        return false;
    }
    bool operator>=( const char* version )
    {
        SoftwarebuildVersion lVersion( version );
        return (*this >= lVersion);
    }
    // Kleiner
    bool operator<( SoftwarebuildVersion& version )
    {
        if ( this->m_master < version.m_master 
            || (this->m_master == version.m_master && this->m_major < version.m_major) 
            || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor < version.m_minor) 
            || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor == version.m_minor && this->m_build < version.m_build) )
        {
            return true;
        }

        return false;
    }
    bool operator<( const char* version )
    {
        SoftwarebuildVersion lVersion( version );
        return (*this < lVersion);
    }
    // Kleiner gleich
    bool operator<=( SoftwarebuildVersion& version )
    {
        if ( *this < version || *this == version )
        {
            return true;
        }

        return false;
    }
    bool operator<=( const char* version )
    {
        SoftwarebuildVersion lVersion( version );
        return (*this <= lVersion);
    }

};
#endif
