/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/
#pragma once
#ifndef DATAFOX_COMMUNICATION_MODULE_TABLE_H
#define DATAFOX_COMMUNICATION_MODULE_TABLE_H

// Basisklasse einbinden.
#include "datafox/communication/module_application.h"

// MODULE_TABLE( 3 )

#define IV_MODULE_TABLE                          0x0003

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter

// Get: Datensatz-/Listenbeschreibung. GET_TABLE( 0 )
#define IV_FUNCTION_TABLE_GET                    0x0000
// (Majorname.Minorname\0)
// (STable2)

// Die Tabellenbeschreibung kann auf folgende Arten ermittelt werden.
// Zum einen ist es noetig die Datensatz-/Listenbeschreibungen des Setup sowie der
// Zutrittskontrolle auseinanderzuhalten. Dann wird entweder der Index oder der Name,
// der jeweilingen Tabellendefinition benoetigt.

// Majornamen:
// RECORD - Sind die Datensatzbeschreibungen
// LIST   - Sind die Listenbeschreibungen
// ACCESS - Sind die Listenbeschreibungen der Zutrittskontrolle

// darauf folgt nach einem Punkt entweder die Nummer oder der Name der Tabelle.

// Get: Datensatz-/Listenbeschreibungsfeld. GET_FIELD( 1 )
#define IV_FUNCTION_TABLE_GET_COLUMN             0x0001
// (Majorname.Minorname.Subminorname\0)
// (STableField)

// Majorname.Minorname bauen sich wie bei den Tabellen auf. Der Subminorname entspricht
// in seinem Aufbau dem Minornamen, ist logischerweise jedoch Feldbezogen.

// Fehler:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = SUBMINOR_NAME ist unbekannt

#define IV_FUNCTION_TABLE_OPEN                         0x0002
//params[in]
// 1. char *name;
// 2. char *reserved;
//params[out]
// 1. int handle;
// 2. char *reserved;

#define IV_FUNCTION_TABLE_CLOSE                        0x0003
// params[int]
// 1. int handle;
// params[out]
// -

#define IV_FUNCTION_TABLE_SET_FILTER                   0x0004
// params[int]
// 1. int handle;
// 2. char *filter;
// params[out]
// -
//-> Aktive Antwort SET_FILTER_OK erwartet.

#define IV_FUNCTION_TABLE_SET_FILTER_OK                0x0005
// params[int]
// -
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_FILTER                   0x0006
// params[int]
// 1. int handle;
// params[out]
// 1. char *filter;

#define IV_FUNCTION_TABLE_REMOVE_FILTER                0x0007
// params[int]
// 1. int handle;
// params[out]
// -

#define IV_FUNCTION_TABLE_SET_CURRENT_ROW              0x0008
// params[int]
// 1. int handle;
// 2. int offset;
// 3. unsigned short direction;
// params[out]
// -

#define IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA_1    0x0009
// params[int]
// 1. int handle;
// 2. char *name;
// 3. unsigned char *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA_1    0x000A
// params[int]
// 1. int handle;
// 2. char *name;
// params[out]
// 1. unsigned char *data;

#define IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA_1       0x000B
// params[int]
// 1. int handle;
// 2. unsinged char *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA_1       0x000C
// params[int]
// 1. int handle;
// params[out]
// 1. unsinged char *data;

#define IV_FUNCTION_TABLE_GENERATE                     0x000D
// params[int]
// 1. char *name;
// 2. char *reserved;
// params[out]
// -
//-> Aktive-Antwort TABLE_GENERATE_OK erwartet.

#define IV_FUCNTION_TABLE_GENERATE_OK                  0x000E
// params[int]
// -
// params[out]
// -

#define IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD    0x000F
// params[int]
// 1. unsigned char type;
// 2. unsigned char *data;
// params[out]
// -
// -> Aktive Antwort APPEND_RECORD_OK erwartet.

#define IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK 0x0010
// params[int]
// -
// params[out]
// -

#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD1      0x0011
// DEPRECATED: Unterstuetzte keine Aktive-Antwort.

#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD1   0x0012
// DEPRECATED: Unterstuetzte keine Aktive-Antwort.

#define IV_FUNCTION_TABLE_GET_ROW_COUNT                0x0013
// params[int]
// 1. int handle;
// params[out]
// 1. unsigned int currentCount;
// 2. unsigned int unsortedCount;
// 3. unsigned int deletedCount;

#define IV_FUNCTION_TABLE_GET_CURRENT_ROW              0x0014
// params[int]
// 1. int handle;
// params[out]
// 1. unsigned int pos;

#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD       0x0015
// params[int]
// 1. unsigned char type;
// 2. unsigned int pid;
// 3. unsigned char fid;
// params[out]
// -
//-> Aktive-Antwort FINGERPRINT_TABLE_GET_RECORD_OK erwartet.

#define IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD_OK    0x0016
// params[int]
// 1. unsigned char type;
// 2. unsigned char *data;
// params[out]
// -

#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD    0x0017
// params[int]
// 1. unsigned int pid;
// 2. unsigned char fid;
// params[out]
// -
//-> Aktive-Antwort FINGERPRINT_TABLE_DELETE_RECORD_OK erwartet.

#define IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD_OK 0x0018
// params[int]
// -
// params[out]
// -

#define IV_FUNCTION_TABLE_APPEND_ROW_DATA              0x0019
// params[int]
// 1. int handle;
// 2. unsigned short flags;
// 3. char separator;
// 4. unsinged char *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_DELETE_CURRENT_ROW           0x001A
// params[int]
// 1. int handle;
// 2. unsigned short flags;
// params[out]
// -

#define IV_FUNCTION_TABLE_SET_CURRENT_ROW_DATA         0x001B
// params[int]
// 1. int handle;
// 2. unsigned short flags;
// 3. char separator;
// 4. unsinged char *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_CURRENT_ROW_DATA         0x001C
// params[int]
// 1. int handle;
// 2. unsigned short flags;
// 3. char separator;
// params[out]
// 1. unsinged char *data;

#define IV_FUNCTION_TABLE_SET_CURRENT_COLUMN_DATA      0x001D
// params[int]
// 1. int handle;
// 2. unsigned short flags;
// 3. char *name;
// 4. unsigned char *data;
// params[out]
// -

#define IV_FUNCTION_TABLE_GET_CURRENT_COLUMN_DATA      0x001E
// params[int]
// 1. int handle;
// 2. unsigned short flags;
// 3. char *name;
// params[out]
// 1. unsigned char *data;

#define IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA  0x001F
// params[int]
// 1. int handle;
// 2. unsigned short flags;
// 3. char *name;
// 4. unsigned char *data;
// params[out]
// -
//-> Aktive-Antwort FINGERPRINT_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA_OK erwartet.

#define IV_FUNCTION_TABLE_SET_ALL_ROWS_TO_COLUMN_DATA_OK   0x0020

#define IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS            0x0021
// params[int]
// 1. int handle;
// 2. unsigned short flags;
// params[out]
// -
//-> Aktive-Antwort IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS_OK erwartet.

#define IV_FUNCTION_TABLE_DELETE_AVAILABLE_ROWS_OK         0x0022

namespace Datafox
{
    namespace Communication
    {
        class Module_table : public Module_application
        {
        public:
            /// Konstruktor fuer Initialisierung der Klasse.
            Module_table( int connection_id = Connection_base::dv_connection_id, uint8_t device_address = Protocol_base::dv_device_address );

            /// Destruktor
            virtual ~Module_table();

        public:
            // Gibt Modulfehler zurueck.
            virtual void error_module( uint16_t module, uint16_t error, std::string &text );
            // Gibt Funktionsfehlertext zurueck.
            virtual void error_function( uint16_t module, uint16_t func, uint16_t error, std::string &text );

            // MODULE_TABLE

            // Tabellendefinition ueber Index oder Namen ermitteln.
            int read_table_struct( unsigned char device_address, const std::string &name, StructTable2 &table );
            // Tabellenfelddefinition ueber Index oder Namen ermitteln.
            int read_table_column_struct( unsigned char device_address, const std::string &name, StructTable2Field &field );

            // Oeffnen einer Tabelle
            int open_table( unsigned char device_address, const std::string &name, unsigned int options, int &handle, std::string &reserved );
            // Schliessen der Tabelle
            int close_table( unsigned char device_address, int handle );

            // Setzen eines Filters auf die Tabelle
            int write_table_filter( unsigned char device_address, int handle, const std::string &filter );
            int reply_write_table_filter( unsigned char device_address );
            // Lesen des Filters auf die Tabelle
            int read_table_filter( unsigned char device_address, int handle, std::string &filter );
            // Entfernen des Filters
            int remove_table_filter( unsigned char device_address, int handle );
    
            // Abfragen der Zeilenanzahl
            int read_table_row_count( unsigned char device_address, int handle, unsigned int &current_count, unsigned int &unsorted_count, unsigned int &deleted_count );
            // Abfragen der aktuellen Datensatznummer
            int read_current_table_row( unsigned char device_address, int handle, unsigned int &current_row );
            // Ueberspringen von n Datensaetzen (offset kann negativ sein), ab top/aktuell/bottom
            int write_current_table_row( unsigned char device_address, int handle, int offset, unsigned short direction );
    
            // Aendern eines aktuellen Datensatzes
            int write_current_table_row_data( unsigned char device_address, int handle, const char* data, unsigned short data_size );
            int write_current_table_row_data_ext( unsigned char device_address, int handle, unsigned short flags, char separator, const char* data, unsigned short data_size );
            // Aendern eines Spaltenwerts, des aktuellen Datensatzes
            int write_current_table_column_data( unsigned char device_address, int handle, const char* name, const char* data );
            int write_current_table_column_data_ext( unsigned char device_address, int handle, unsigned short flags, const char* name, const char* data, unsigned short data_size );
            int write_all_table_rows_column_data( unsigned char device_address, int handle, unsigned short flags, const char* name, const char* data, unsigned short data_size );
            int reply_write_all_table_rows_column_data( unsigned char device_address );
    
            // Abfragen eines aktuellen Datensatzes
            int read_current_table_row_data( unsigned char device_address, int handle, char* data, unsigned short &data_size );
            int read_current_table_row_data_ext( unsigned char device_address, int handle, unsigned short flags, char separator, char* data, unsigned short &data_size );
            // Abfragen eines Spaltenwerts, des aktuellen Datensatzes
            int read_current_table_column_data( unsigned char device_address, int handle, const char* name, char* data, unsigned short &data_size );
            int read_current_table_column_data_ext( unsigned char device_address, int handle, unsigned short flags, const char* name, char* data, unsigned short &data_size );

            // Daten an Listentabelle anfuegen.
            int append_table_row_data( unsigned char device_address, int handle, unsigned short flags, char separator, const char* data, unsigned short data_size );
            // Loeschen des aktuellen Datensatzes
            int delete_current_table_row( unsigned char device_address, int handle, unsigned short flags );
            // Loeschen der Datensaetze die durch den gesetzten Filter erreichbar sind.
            int delete_filtrated_table_rows( unsigned char device_address, int handle, unsigned short flags );
            int reply_delete_filtrated_table_rows( unsigned char device_address );

            // Tabelle erzeugen
            int generate_table( unsigned char device_address, const std::string &name, const std::string &reserved );
            int reply_generate_table( unsigned char device_address );

            // Datensatz an Fingerprint-Tabelle anfuegen.
            int append_fingerprint_record( unsigned char device_address, unsigned char type, const unsigned char* data, int size );
            int reply_append_fingerprint_record( unsigned char device_address );
            // Datensatz aus Fingerprint-Tabelle lesen.
            int read_fingerprint_record( unsigned char device_address, unsigned char &type, unsigned int pid, unsigned char fid, unsigned char* data, int &size );
            int reply_read_fingerprint_record( unsigned char device_address );
            // Datensatz aus Fingerprint-Tabelle loeschen.
            int delete_fingerprint_record( unsigned char device_address, unsigned int pid, unsigned char fid );
            int reply_delete_fingerprint_record( unsigned char device_address );

            //#################################
            // CallRoutinen

            // Ausfuehren von Befehlen
            virtual int remote_call( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count );

        };

    } // namespace Communication
} // namespace Datafox

#endif
