/**
* @file  file_wlan_config.h
* @brief Schnittstelle fuer die Klasse FileWlanConfig.
*/
//////////////////////////////////////////////////////////////////////
#pragma once
#ifndef FILE_WLAN_CONFIG_H
#define FILE_WLAN_CONFIG_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

// Verwaltungsklasse fuer Datei
#include "file_intel_hex.h"
// Verschluesselung
#include "common/df4_crypt.h"

// Strukturdefinitionen
#include "interface/wlan_config_structs.h"

#include <array>

class FileWlanConfig : public FileIntelHex
{
public:
    FileWlanConfig();
    virtual ~FileWlanConfig();

public:
    enum { max_wlan_configs = 8 };
    // Verwaltungsstrukturen fuer WlanConfig
    WLANCONFIGNODE *m_wlan_config_node;
    std::array<WLANCONFIG *, max_wlan_configs> m_wlan_config;

public:
    void resetData();
    void init();
    bool load( const std::string &filePath );
    bool save( const std::string &filePath, bool appendToFile = false, bool appendEndLine = true );

    static void DefaultWlanConfig( WLANCONFIG &wc );
    void SetWlanConfig( int idx, const WLANCONFIG &wc );
    WLANCONFIG GetWlanConfig( int idx );
    int GetWlanConfigCount()
    {
        return m_wlan_config_node->cnt;
    }
    void WlanConfigCRC( int idx );
    bool WlanConfigCheckCRC( int idx );
    void WlanConfigCrypt( int idx );
    bool isModified() 
    {
        return (m_crc != checksum32bit());
    }

private:
    unsigned int m_crc;
    df4_crypt m_crypt;
};

#endif

